@@ -24,27 +24,35 @@
  * document from.
  */
 
-#include <time.h>
-#include <string.h>
-
-#include "geany.h"
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
 
 #include "templates.h"
-#include "support.h"
-#include "utils.h"
+
+#include "app.h"
 #include "document.h"
 #include "encodings.h"
-#include "editor.h"
 #include "filetypes.h"
-#include "ui_utils.h"
-#include "toolbar.h"
+#include "geany.h"
 #include "geanymenubuttonaction.h"
-#include "project.h"
+#include "geanyobject.h"
+#include "spawn.h"
+#include "support.h"
+#include "toolbar.h"
+#include "ui_utils.h"
+#include "utils.h"
+
+#include "gtkcompat.h"
+
+#include <time.h>
+#include <string.h>
 
 
 GeanyTemplatePrefs template_prefs;
 
-static GtkWidget *new_with_template_menu = NULL;	/* submenu used for both file menu and toolbar */
+static GtkWidget *new_with_template_menu = NULL;
+static GtkWidget *new_with_template_toolbar_menu = NULL;
 
 /* TODO: implement custom insertion templates instead? */
 static gchar *templates[GEANY_MAX_TEMPLATES];
@@ -107,6 +115,8 @@ static void convert_eol_characters(GString *template, GeanyDocument *doc)
 {
 	gint doc_eol_mode;
 
+	g_return_if_fail(doc == NULL || doc->is_valid);
+
 	if (doc == NULL)
 		doc = document_get_current();
 
@@ -249,7 +259,7 @@ static void add_file_item(const gchar *fname, GtkWidget *menu)
 }
 
 
-static gboolean add_custom_template_items(void)
+static void populate_file_template_menu(GtkWidget *menu)
 {
 	GSList *list = utils_get_config_files(GEANY_TEMPLATES_SUBDIR G_DIR_SEPARATOR_S "files");
 	GSList *node;
@@ -258,50 +268,36 @@ static gboolean add_custom_template_items(void)
 	{
 		gchar *fname = node->data;
 
-		add_file_item(fname, new_with_template_menu);
+		add_file_item(fname, menu);
 		g_free(fname);
 	}
 	g_slist_free(list);
-	return list != NULL;
 }
 
 
 static void create_file_template_menu(void)
 {
+	GtkWidget *item;
+
 	new_with_template_menu = gtk_menu_new();
-	add_custom_template_items();
-
-	/* unless the file menu is showing, menu should be in the toolbar widget */
-	geany_menu_button_action_set_menu(GEANY_MENU_BUTTON_ACTION(
-		toolbar_get_action_by_name("New")), new_with_template_menu);
-}
-
-
-static void on_file_menu_show(GtkWidget *item)
-{
-	geany_menu_button_action_set_menu(
-		GEANY_MENU_BUTTON_ACTION(toolbar_get_action_by_name("New")), NULL);
 	item = ui_lookup_widget(main_widgets.window, "menu_new_with_template1");
 	gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), new_with_template_menu);
-}
 
-
-static void on_file_menu_hide(GtkWidget *item)
-{
-	item = ui_lookup_widget(main_widgets.window, "menu_new_with_template1");
-	gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), NULL);
-	geany_menu_button_action_set_menu(
-		GEANY_MENU_BUTTON_ACTION(toolbar_get_action_by_name("New")), new_with_template_menu);
+	new_with_template_toolbar_menu = gtk_menu_new();
+	g_object_ref(new_with_template_toolbar_menu);
+	geany_menu_button_action_set_menu(GEANY_MENU_BUTTON_ACTION(toolbar_get_action_by_name("New")), 
+		new_with_template_toolbar_menu);
 }
 
 
 /* reload templates if any file in the templates path is saved */
 static void on_document_save(G_GNUC_UNUSED GObject *object, GeanyDocument *doc)
 {
-	gchar *path = g_build_filename(app->configdir, GEANY_TEMPLATES_SUBDIR, NULL);
+	gchar *path;
 
-	g_return_if_fail(NZV(doc->real_path));
+	g_return_if_fail(!EMPTY(doc->real_path));
 
+	path = g_build_filename(app->configdir, GEANY_TEMPLATES_SUBDIR, NULL);
 	if (strncmp(doc->real_path, path, strlen(path)) == 0)
 	{
 		/* reload templates */
@@ -319,23 +315,15 @@ void templates_init(void)
 
 	init_general_templates();
 
-	create_file_template_menu();
-	/* we hold our own ref for the menu as it has no parent whilst being moved */
-	g_object_ref(new_with_template_menu);
-
-	/* only connect signals to persistent objects once */
 	if (!init_done)
 	{
-		GtkWidget *item;
-		/* reparent the template menu as needed */
-		item = ui_lookup_widget(main_widgets.window, "file1");
-		item = gtk_menu_item_get_submenu(GTK_MENU_ITEM(item));
-		g_signal_connect(item, "show", G_CALLBACK(on_file_menu_show), NULL);
-		g_signal_connect(item, "hide", G_CALLBACK(on_file_menu_hide), NULL);
-
+		create_file_template_menu();
 		g_signal_connect(geany_object, "document-save", G_CALLBACK(on_document_save), NULL);
+		init_done = TRUE;
 	}
-	init_done = TRUE;
+
+	populate_file_template_menu(new_with_template_menu);
+	populate_file_template_menu(new_with_template_toolbar_menu);
 }
 
 
@@ -365,9 +353,9 @@ static void make_comment_block(GString *comment_text, gint filetype_idx, guint i
 	template_eol_char = utils_get_eol_char(template_eol_mode);
 
 	filetype_get_comment_open_close(ft, FALSE, &co, &cc);
-	if (NZV(co))
+	if (!EMPTY(co))
 	{
-		if (NZV(cc))
+		if (!EMPTY(cc))
 		{
 			frame_start = g_strconcat(co, template_eol_char, NULL);
 			frame_end = g_strconcat(cc, template_eol_char, NULL);
@@ -388,7 +376,7 @@ static void make_comment_block(GString *comment_text, gint filetype_idx, guint i
 	}
 
 	/* do some magic to nicely format C-like multi-line comments */
-	if (NZV(frame_start) && frame_start[1] == '*')
+	if (!EMPTY(frame_start) && frame_start[1] == '*')
 	{
 		/* prefix the string with a space */
 		SETPTR(frame_end, g_strconcat(" ", frame_end, NULL));
@@ -436,7 +424,7 @@ gchar *templates_get_template_licence(GeanyDocument *doc, gint licence_type)
 {
 	GString *template;
 
-	g_return_val_if_fail(doc != NULL, NULL);
+	g_return_val_if_fail(DOC_VALID(doc), NULL);
 	g_return_val_if_fail(licence_type == GEANY_TEMPLATE_GPL || licence_type == GEANY_TEMPLATE_BSD, NULL);
 
 	template = g_string_new(templates[licence_type]);
@@ -469,6 +457,7 @@ static gchar *get_template_fileheader(GeanyFiletype *ft)
 
 
 /* TODO change the signature to take a GeanyDocument? this would break plugin API/ABI */
+GEANY_API_SYMBOL
 gchar *templates_get_template_fileheader(gint filetype_idx, const gchar *fname)
 {
 	GeanyFiletype *ft = filetypes[filetype_idx];
@@ -502,9 +491,13 @@ gchar *templates_get_template_function(GeanyDocument *doc, const gchar *func_nam
 
 gchar *templates_get_template_changelog(GeanyDocument *doc)
 {
-	GString *result = g_string_new(templates[GEANY_TEMPLATE_CHANGELOG]);
-	const gchar *file_type_name = (doc != NULL) ? doc->file_type->name : "";
+	GString *result;
+	const gchar *file_type_name;
 
+	g_return_val_if_fail(DOC_VALID(doc), NULL);
+
+	result = g_string_new(templates[GEANY_TEMPLATE_CHANGELOG]);
+	file_type_name = (doc->file_type != NULL) ? doc->file_type->name : "";
 	replace_static_values(result);
 	templates_replace_default_dates(result);
 	templates_replace_command(result, DOC_FILENAME(doc), file_type_name, NULL);
@@ -514,30 +507,25 @@ gchar *templates_get_template_changelog(GeanyDocument *doc)
 }
 
 
+static void free_template_menu_items(GtkWidget *menu)
+{
+	GList *children, *item;
+
+	children = gtk_container_get_children(GTK_CONTAINER(menu));
+	foreach_list(item, children)
+		gtk_widget_destroy(GTK_WIDGET(item->data));
+	g_list_free(children);
+}
+
+
 void templates_free_templates(void)
 {
 	gint i;
-	GList *children, *item;
-
-	/* disconnect the menu from the action widget, so destroying the items below doesn't
-	 * trigger rebuilding of the menu on each item destroy */
-	geany_menu_button_action_set_menu(
-		GEANY_MENU_BUTTON_ACTION(toolbar_get_action_by_name("New")), NULL);
 
 	for (i = 0; i < GEANY_MAX_TEMPLATES; i++)
-	{
 		g_free(templates[i]);
-	}
-	/* destroy "New with template" sub menu items (in case we want to reload the templates) */
-	children = gtk_container_get_children(GTK_CONTAINER(new_with_template_menu));
-	foreach_list(item, children)
-	{
-		gtk_widget_destroy(GTK_WIDGET(item->data));
-	}
-	g_list_free(children);
-
-	g_object_unref(new_with_template_menu);
-	new_with_template_menu = NULL;
+	free_template_menu_items(new_with_template_menu);
+	free_template_menu_items(new_with_template_toolbar_menu);
 }
 
 
@@ -608,33 +596,33 @@ static void templates_replace_default_dates(GString *text)
 static gchar *run_command(const gchar *command, const gchar *file_name,
 						  const gchar *file_type, const gchar *func_name)
 {
+	GString *output = g_string_new(NULL);
 	gchar *result = NULL;
-	gchar **argv;
+	GError *error = NULL;
+	gchar **env;
 
-	if (g_shell_parse_argv(command, NULL, &argv, NULL))
+	file_name = (file_name != NULL) ? file_name : "";
+	file_type = (file_type != NULL) ? file_type : "";
+	func_name = (func_name != NULL) ? func_name : "";
+
+	env = utils_copy_environment(NULL,
+		"GEANY_FILENAME", file_name,
+		"GEANY_FILETYPE", file_type,
+		"GEANY_FUNCNAME", func_name,
+		NULL);
+
+	if (spawn_sync(NULL, command, NULL, env, NULL, output, NULL, NULL, &error))
 	{
-		GError *error = NULL;
-		gchar **env;
-
-		file_name = (file_name != NULL) ? file_name : "";
-		file_type = (file_type != NULL) ? file_type : "";
-		func_name = (func_name != NULL) ? func_name : "";
-
-		env = utils_copy_environment(NULL,
-			"GEANY_FILENAME", file_name,
-			"GEANY_FILETYPE", file_type,
-			"GEANY_FUNCNAME", func_name,
-			NULL);
-		if (! utils_spawn_sync(NULL, argv, env, G_SPAWN_SEARCH_PATH,
-				NULL, NULL, &result, NULL, NULL, &error))
-		{
-			g_warning("templates_replace_command: %s", error->message);
-			g_error_free(error);
-			return NULL;
-		}
-		g_strfreev(argv);
-		g_strfreev(env);
+		result = g_string_free(output, FALSE);
 	}
+	else
+	{
+		g_warning(_("Cannot execute command \"%s\" from the template: %s. "
+			"Check the path in the template."), command, error->message);
+		g_error_free(error);
+	}
+
+	g_strfreev(env);
 	return result;
 }
 
