@@ -24,32 +24,28 @@
  * For Plugins code see plugins.c.
  */
 
-#include "geany.h"
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
+#include "tools.h"
+
+#include "document.h"
+#include "keybindings.h"
+#include "sciwrappers.h"
+#include "spawn.h"
+#include "support.h"
+#include "ui_utils.h"
+#include "utils.h"
+#include "win32.h"
+
+#include "gtkcompat.h"
 
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
 #include <errno.h>
 
-#ifdef G_OS_UNIX
-# include <sys/types.h>
-# include <sys/wait.h>
-# include <signal.h>
-#endif
-
-#include "tools.h"
-#include "support.h"
-#include "document.h"
-#include "editor.h"
-#include "sciwrappers.h"
-#include "utils.h"
-#include "ui_utils.h"
-#include "msgwindow.h"
-#include "keybindings.h"
-#include "templates.h"
-#include "win32.h"
-#include "dialogs.h"
-
 
 enum
 {
@@ -75,20 +71,6 @@ struct cc_dialog
 	GtkWidget *button_down;
 };
 
-static gboolean cc_error_occurred = FALSE;
-static gboolean cc_reading_finished = FALSE;
-static GString *cc_buffer;
-
-
-static gboolean cc_exists_command(const gchar *command)
-{
-	gchar *path = g_find_program_in_path(command);
-
-	g_free(path);
-
-	return path != NULL;
-}
-
 
 /* update STATUS and TOOLTIP columns according to cmd */
 static void cc_dialog_update_row_status(GtkListStore *store, GtkTreeIter *iter, const gchar *cmd)
@@ -96,19 +78,9 @@ static void cc_dialog_update_row_status(GtkListStore *store, GtkTreeIter *iter,
 	GError *err = NULL;
 	const gchar *stock_id = GTK_STOCK_NO;
 	gchar *tooltip = NULL;
-	gint argc;
-	gchar **argv;
 
-	if (! NZV(cmd))
+	if (EMPTY(cmd) || spawn_check_command(cmd, TRUE, &err))
 		stock_id = GTK_STOCK_YES;
-	else if (g_shell_parse_argv(cmd, &argc, &argv, &err))
-	{
-		if (argc > 0 && cc_exists_command(argv[0]))
-			stock_id = GTK_STOCK_YES;
-		else
-			tooltip = g_strdup_printf(_("Invalid command: %s"), _("Command not found"));
-		g_strfreev(argv);
-	}
 	else
 	{
 		tooltip = g_strdup_printf(_("Invalid command: %s"), err->message);
@@ -157,12 +129,28 @@ static void cc_on_dialog_add_clicked(GtkButton *button, struct cc_dialog *cc)
 }
 
 
+static void scroll_to_cursor(GtkTreeView *view)
+{
+	GtkTreePath *path;
+	GtkTreeViewColumn *column;
+
+	gtk_tree_view_get_cursor(view, &path, &column);
+	if (path)
+	{
+		gtk_tree_view_scroll_to_cell(view, path, column, FALSE, 1.0, 1.0);
+		gtk_tree_path_free(path);
+	}
+}
+
 static void cc_on_dialog_remove_clicked(GtkButton *button, struct cc_dialog *cc)
 {
 	GtkTreeIter iter;
 
 	if (gtk_tree_selection_get_selected(cc->selection, NULL, &iter))
+	{
 		gtk_list_store_remove(cc->store, &iter);
+		scroll_to_cursor(GTK_TREE_VIEW(cc->view));
+	}
 }
 
 
@@ -180,6 +168,7 @@ static void cc_on_dialog_move_up_clicked(GtkButton *button, struct cc_dialog *cc
 			gtk_tree_model_get_iter(GTK_TREE_MODEL(cc->store), &prev, path))
 		{
 			gtk_list_store_move_before(cc->store, &iter, &prev);
+			scroll_to_cursor(GTK_TREE_VIEW(cc->view));
 		}
 		gtk_tree_path_free(path);
 	}
@@ -195,140 +184,11 @@ static void cc_on_dialog_move_down_clicked(GtkButton *button, struct cc_dialog *
 		GtkTreeIter next = iter;
 
 		if (gtk_tree_model_iter_next(GTK_TREE_MODEL(cc->store), &next))
+		{
 			gtk_list_store_move_after(cc->store, &iter, &next);
-	}
-}
-
-
-static gboolean cc_iofunc(GIOChannel *ioc, GIOCondition cond, gpointer data)
-{
-	if (cond & (G_IO_IN | G_IO_PRI))
-	{
-		gchar *msg = NULL;
-		GIOStatus rv;
-		GError *err = NULL;
-
-		cc_buffer = g_string_sized_new(256);
-
-		do
-		{
-			rv = g_io_channel_read_line(ioc, &msg, NULL, NULL, &err);
-			if (msg != NULL)
-			{
-				g_string_append(cc_buffer, msg);
-				g_free(msg);
-			}
-			if (G_UNLIKELY(err != NULL))
-			{
-				geany_debug("%s: %s", G_STRFUNC, err->message);
-				g_error_free(err);
-				err = NULL;
-			}
-		} while (rv == G_IO_STATUS_NORMAL || rv == G_IO_STATUS_AGAIN);
-
-		if (G_UNLIKELY(rv != G_IO_STATUS_EOF))
-		{	/* Something went wrong? */
-			g_warning("%s: %s\n", G_STRFUNC, "Incomplete command output");
+			scroll_to_cursor(GTK_TREE_VIEW(cc->view));
 		}
 	}
-	return FALSE;
-}
-
-
-static gboolean cc_iofunc_err(GIOChannel *ioc, GIOCondition cond, gpointer data)
-{
-	if (cond & (G_IO_IN | G_IO_PRI))
-	{
-		gchar *msg = NULL;
-		GString *str = g_string_sized_new(256);
-		GIOStatus rv;
-
-		do
-		{
-			rv = g_io_channel_read_line(ioc, &msg, NULL, NULL, NULL);
-			if (msg != NULL)
-			{
-				g_string_append(str, msg);
-				g_free(msg);
-			}
-		} while (rv == G_IO_STATUS_NORMAL || rv == G_IO_STATUS_AGAIN);
-
-		if (NZV(str->str))
-		{
-			g_warning("%s: %s\n", (const gchar *) data, str->str);
-			ui_set_statusbar(TRUE,
-				_("The executed custom command returned an error. "
-				"Your selection was not changed. Error message: %s"),
-				str->str);
-			cc_error_occurred = TRUE;
-
-		}
-		g_string_free(str, TRUE);
-	}
-	cc_reading_finished = TRUE;
-	return FALSE;
-}
-
-
-static gboolean cc_replace_sel_cb(gpointer user_data)
-{
-	GeanyDocument *doc = user_data;
-
-	if (! cc_reading_finished)
-	{	/* keep this function in the main loop until cc_iofunc_err() has finished */
-		return TRUE;
-	}
-
-	if (! cc_error_occurred && cc_buffer != NULL)
-	{	/* Command completed successfully */
-		sci_replace_sel(doc->editor->sci, cc_buffer->str);
-		g_string_free(cc_buffer, TRUE);
-		cc_buffer = NULL;
-	}
-
-	cc_error_occurred = FALSE;
-	cc_reading_finished = FALSE;
-
-	return FALSE;
-}
-
-
-/* check whether the executed command failed and if so do nothing.
- * If it returned with a sucessful exit code, replace the selection. */
-static void cc_exit_cb(GPid child_pid, gint status, gpointer user_data)
-{
-	/* if there was already an error, skip further checks */
-	if (! cc_error_occurred)
-	{
-#ifdef G_OS_UNIX
-		if (WIFEXITED(status))
-		{
-			if (WEXITSTATUS(status) != EXIT_SUCCESS)
-				cc_error_occurred = TRUE;
-		}
-		else if (WIFSIGNALED(status))
-		{	/* the terminating signal: WTERMSIG (status)); */
-			cc_error_occurred = TRUE;
-		}
-		else
-		{	/* any other failure occured */
-			cc_error_occurred = TRUE;
-		}
-#else
-		cc_error_occurred = ! win32_get_exit_status(child_pid);
-#endif
-
-		if (cc_error_occurred)
-		{	/* here we are sure cc_error_occurred was set due to an unsuccessful exit code
-			 * and so we add an error message */
-			/* TODO maybe include the exit code in the error message */
-			ui_set_statusbar(TRUE,
-				_("The executed custom command exited with an unsuccessful exit code."));
-		}
-	}
-
-	g_idle_add(cc_replace_sel_cb, user_data);
-	g_spawn_close_pid(child_pid);
 }
 
 
@@ -338,72 +198,55 @@ static void cc_exit_cb(GPid child_pid, gint status, gpointer user_data)
 void tools_execute_custom_command(GeanyDocument *doc, const gchar *command)
 {
 	GError *error = NULL;
-	GPid pid;
-	gchar **argv;
-	gint stdin_fd;
-	gint stdout_fd;
-	gint stderr_fd;
+	gchar *sel;
+	SpawnWriteData input;
+	GString *output;
+	GString *errors;
+	gint status;
 
 	g_return_if_fail(doc != NULL && command != NULL);
-
+ 
 	if (! sci_has_selection(doc->editor->sci))
 		editor_select_lines(doc->editor, FALSE);
 
-	if (!g_shell_parse_argv(command, NULL, &argv, &error))
-	{
-		ui_set_statusbar(TRUE, _("Custom command failed: %s"), error->message);
-		g_error_free(error);
-		return;
-	}
+	sel = sci_get_selection_contents(doc->editor->sci);
+	input.ptr = sel;
+	input.size = strlen(sel);
+	output = g_string_sized_new(256);
+	errors = g_string_new(NULL);
 	ui_set_statusbar(TRUE, _("Passing data and executing custom command: %s"), command);
 
-	cc_error_occurred = FALSE;
-
-	if (g_spawn_async_with_pipes(NULL, argv, NULL, G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD,
-						NULL, NULL, &pid, &stdin_fd, &stdout_fd, &stderr_fd, &error))
+	if (spawn_sync(NULL, command, NULL, NULL, &input, output, errors, &status, &error))
 	{
-		gchar *sel;
-		gint len, remaining, wrote;
-
-		if (pid != 0)
-			g_child_watch_add(pid, (GChildWatchFunc) cc_exit_cb, doc);
-
-		/* use GIOChannel to monitor stdout */
-		utils_set_up_io_channel(stdout_fd, G_IO_IN | G_IO_PRI | G_IO_ERR | G_IO_HUP | G_IO_NVAL,
-				FALSE, cc_iofunc, NULL);
-		/* copy program's stderr to Geany's stdout to help error tracking */
-		utils_set_up_io_channel(stderr_fd, G_IO_IN | G_IO_PRI | G_IO_ERR | G_IO_HUP | G_IO_NVAL,
-				FALSE, cc_iofunc_err, (gpointer)command);
-
-		/* get selection */
-		len = sci_get_selected_text_length(doc->editor->sci);
-		sel = g_malloc0(len + 1);
-		sci_get_selected_text(doc->editor->sci, sel);
-
-		/* write data to the command */
-		remaining = len - 1;
-		do
+		if (errors->len > 0)
 		{
-			wrote = write(stdin_fd, sel, remaining);
-			if (G_UNLIKELY(wrote < 0))
-			{
-				g_warning("%s: %s: %s\n", G_STRFUNC, "Failed sending data to command",
-										g_strerror(errno));
-				break;
-			}
-			remaining -= wrote;
-		} while (remaining > 0);
-		close(stdin_fd);
-		g_free(sel);
+			g_warning("%s: %s\n", command, errors->str);
+			ui_set_statusbar(TRUE,
+				_("The executed custom command returned an error. "
+				"Your selection was not changed. Error message: %s"),
+				errors->str);
+		}
+		else if (!SPAWN_WIFEXITED(status) || SPAWN_WEXITSTATUS(status) != EXIT_SUCCESS)
+		{
+			/* TODO maybe include the exit code in the error message */
+			ui_set_statusbar(TRUE,
+				_("The executed custom command exited with an unsuccessful exit code."));
+		}
+		else
+		{   /* Command completed successfully */
+			sci_replace_sel(doc->editor->sci, output->str);
+		}
 	}
 	else
 	{
-		geany_debug("g_spawn_async_with_pipes() failed: %s", error->message);
-		ui_set_statusbar(TRUE, _("Custom command failed: %s"), error->message);
+		ui_set_statusbar(TRUE, _("Cannot execute custom command \"%s\": %s. "
+			"Check the path setting in Custom Commands."), command, error->message);
 		g_error_free(error);
 	}
 
-	g_strfreev(argv);
+	g_string_free(output, TRUE);
+	g_string_free(errors, TRUE);
+	g_free(sel);
 }
 
 
@@ -528,7 +371,7 @@ static void cc_show_dialog_custom_commands(void)
 	cc.store = gtk_list_store_new(CC_COLUMN_COUNT, G_TYPE_UINT, G_TYPE_STRING, G_TYPE_STRING,
 			G_TYPE_STRING, G_TYPE_STRING);
 	cc.view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(cc.store));
-	gtk_tree_view_set_tooltip_column(GTK_TREE_VIEW(cc.view), CC_COLUMN_TOOLTIP);
+	ui_tree_view_set_tooltip_text_column(GTK_TREE_VIEW(cc.view), CC_COLUMN_TOOLTIP);
 	gtk_tree_view_set_reorderable(GTK_TREE_VIEW(cc.view), TRUE);
 	cc.selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(cc.view));
 	/* ID column */
@@ -569,7 +412,7 @@ static void cc_show_dialog_custom_commands(void)
 
 		for (i = 0; i < len; i++)
 		{
-			if (! NZV(ui_prefs.custom_commands[i]))
+			if (EMPTY(ui_prefs.custom_commands[i]))
 				continue; /* skip empty fields */
 
 			cc_dialog_add_command(&cc, i, FALSE);
@@ -628,7 +471,7 @@ static void cc_show_dialog_custom_commands(void)
 				gchar *lbl;
 
 				gtk_tree_model_get(GTK_TREE_MODEL(cc.store), &iter, CC_COLUMN_CMD, &cmd, CC_COLUMN_LABEL, &lbl, -1);
-				if (NZV(cmd))
+				if (!EMPTY(cmd))
 				{
 					cmd_list = g_slist_prepend(cmd_list, cmd);
 					lbl_list = g_slist_prepend(lbl_list, lbl);
@@ -683,7 +526,7 @@ static void cc_on_custom_command_activate(GtkMenuItem *menuitem, gpointer user_d
 	GeanyDocument *doc = document_get_current();
 	gint command_idx;
 
-	g_return_if_fail(doc != NULL);
+	g_return_if_fail(DOC_VALID(doc));
 
 	command_idx = GPOINTER_TO_INT(user_data);
 
@@ -718,8 +561,11 @@ static void cc_insert_custom_command_items(GtkMenu *me, const gchar *label, cons
 	if (key_idx != -1)
 	{
 		kb = keybindings_lookup_item(GEANY_KEY_GROUP_FORMAT, key_idx);
-		gtk_widget_add_accelerator(item, "activate", gtk_accel_group_new(),
-			kb->key, kb->mods, GTK_ACCEL_VISIBLE);
+		if (kb->key > 0)
+		{
+			gtk_widget_add_accelerator(item, "activate", gtk_accel_group_new(),
+				kb->key, kb->mods, GTK_ACCEL_VISIBLE);
+		}
 	}
 	gtk_container_add(GTK_CONTAINER(me), item);
 	gtk_widget_show(item);
@@ -756,9 +602,9 @@ void tools_create_insert_custom_command_menu_items(void)
 		{
 			const gchar *label = ui_prefs.custom_commands_labels[i];
 
-			if (! NZV(label))
+			if (EMPTY(label))
 				label = ui_prefs.custom_commands[i];
-			if (NZV(label)) /* skip empty items */
+			if (!EMPTY(label)) /* skip empty items */
 			{
 				cc_insert_custom_command_items(menu_edit, label, ui_prefs.custom_commands[i], idx);
 				idx++;
@@ -929,19 +775,20 @@ void tools_word_count(void)
 /*
  * color dialog callbacks
  */
-#ifndef G_OS_WIN32
 static void on_color_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 {
 	switch (response)
 	{
 		case GTK_RESPONSE_OK:
+			gtk_widget_hide(ui_widgets.open_colorsel);
+			/* fall through */
+		case GTK_RESPONSE_APPLY:
 		{
 			GdkColor color;
 			GeanyDocument *doc = document_get_current();
 			gchar *hex;
 			GtkWidget *colorsel;
 
-			gtk_widget_hide(ui_widgets.open_colorsel);
 			g_return_if_fail(doc != NULL);
 
 			colorsel = gtk_color_selection_dialog_get_color_selection(GTK_COLOR_SELECTION_DIALOG(ui_widgets.open_colorsel));
@@ -957,21 +804,28 @@ static void on_color_dialog_response(GtkDialog *dialog, gint response, gpointer
 			gtk_widget_hide(ui_widgets.open_colorsel);
 	}
 }
-#endif
 
 
 /* This shows the color selection dialog to choose a color. */
 void tools_color_chooser(const gchar *color)
 {
-#ifdef G_OS_WIN32
-	win32_show_color_dialog(color);
-#else
-	gchar *c = (gchar*) color;
+	GdkColor gc;
 	GtkWidget *colorsel;
 
+#ifdef G_OS_WIN32
+	if (interface_prefs.use_native_windows_dialogs)
+	{
+		win32_show_color_dialog(color);
+		return;
+	}
+#endif
+
 	if (ui_widgets.open_colorsel == NULL)
 	{
 		ui_widgets.open_colorsel = gtk_color_selection_dialog_new(_("Color Chooser"));
+		gtk_dialog_add_button(GTK_DIALOG(ui_widgets.open_colorsel), GTK_STOCK_APPLY, GTK_RESPONSE_APPLY);
+		ui_dialog_set_primary_button_order(GTK_DIALOG(ui_widgets.open_colorsel),
+				GTK_RESPONSE_APPLY, GTK_RESPONSE_CANCEL, GTK_RESPONSE_OK, -1);
 		gtk_widget_set_name(ui_widgets.open_colorsel, "GeanyDialog");
 		gtk_window_set_transient_for(GTK_WINDOW(ui_widgets.open_colorsel), GTK_WINDOW(main_widgets.window));
 		colorsel = gtk_color_selection_dialog_get_color_selection(GTK_COLOR_SELECTION_DIALOG(ui_widgets.open_colorsel));
@@ -985,21 +839,12 @@ void tools_color_chooser(const gchar *color)
 	else
 		colorsel = gtk_color_selection_dialog_get_color_selection(GTK_COLOR_SELECTION_DIALOG(ui_widgets.open_colorsel));
 	/* if color is non-NULL set it in the dialog as preselected color */
-	if (c != NULL && (c[0] == '0' || c[0] == '#'))
+	if (color != NULL && utils_parse_color(color, &gc))
 	{
-		GdkColor gc;
-
-		if (c[0] == '0' && c[1] == 'x')
-		{	/* we have a string of the format "0x00ff00" and we need it to "#00ff00" */
-			c[1] = '#';
-			c++;
-		}
-		gdk_color_parse(c, &gc);
 		gtk_color_selection_set_current_color(GTK_COLOR_SELECTION(colorsel), &gc);
 		gtk_color_selection_set_previous_color(GTK_COLOR_SELECTION(colorsel), &gc);
 	}
 
 	/* We make sure the dialog is visible. */
 	gtk_window_present(GTK_WINDOW(ui_widgets.open_colorsel));
-#endif
 }
